//
//  WiFiConnect.m
//  WiMaster_Test_iPhne4
//
//  Created by 石井 孝佳 on 13/08/20.
//  Copyright (c) 2013年 石井 孝佳. All rights reserved.
//

#import "WiFiConnect.h"

@implementation WiFiConnect

@synthesize delegate;

//////////////////////////////////////////////////////////////
// コンストラクタ
//////////////////////////////////////////////////////////////
-(id)initWithHostName:(NSString *)hostName port:(UInt32)port timeout:(int)timeoutSec{
    self = [super init];
    if(self){
        _hostName = hostName;
        _port = port;
        _readStream = nil;
        _writeStream = nil;
    }
    return self;
}


//////////////////////////////////////////////////////////////
//  ソケットオープン
//////////////////////////////////////////////////////////////
-(BOOL)openSocket
{
    //CFStreamで開いて、NSStreamに渡す
    CFReadStreamRef readStream;
    CFWriteStreamRef writeStream;
    
    CFStreamCreatePairWithSocketToHost(kCFAllocatorDefault, (__bridge CFStringRef)_hostName,  _port, &readStream,&writeStream);
    
    _readStream = (__bridge_transfer NSInputStream*)readStream;
    _readStream.delegate = self;
    [_readStream scheduleInRunLoop:[NSRunLoop mainRunLoop] forMode:NSDefaultRunLoopMode];
    
    _writeStream = (__bridge_transfer NSOutputStream*)writeStream;
    _writeStream.delegate = self;
    [_writeStream scheduleInRunLoop:[NSRunLoop mainRunLoop] forMode:NSDefaultRunLoopMode];
    
    [_readStream open];
    [_writeStream open];
    
    return YES;
}


//////////////////////////////////////////////////////////////
//  ソケットクローズ
//////////////////////////////////////////////////////////////
-(void)closeSocket
{
    if(_writeStream){
        _writeStream.delegate = nil;
        [_writeStream close];
        [_writeStream removeFromRunLoop:[NSRunLoop mainRunLoop] forMode:NSDefaultRunLoopMode];
        _writeStream = nil;
    }
    if(_readStream){
        _readStream.delegate = nil;
        [_readStream close];
        [_readStream removeFromRunLoop:[NSRunLoop mainRunLoop] forMode:NSDefaultRunLoopMode];
        _readStream = nil;
    }
}


//////////////////////////////////////////////////////////////
//  データ送信
//////////////////////////////////////////////////////////////
- (bool)writeData:(const void*)data length:(NSUInteger)len
{
    BOOL ret = NO;
    NSInteger leftlen = len;
    if(leftlen <= 0) return YES;
    while(TRUE){
        NSStreamStatus stat = _writeStream.streamStatus;
        if(stat == NSStreamStatusOpen || stat == NSStreamStatusWriting){
            if([_writeStream hasSpaceAvailable]){
                NSInteger count = [_writeStream write:(data + (len - leftlen)) maxLength:leftlen];
                if(count >= 0){
                    leftlen -= count;
                    if(leftlen <= 0){
                        ret = YES;
                        break;
                    }
                }else{
                    NSLog(@"writeData error %@",_writeStream.streamError.description);
                    break;
                }
            }
        }else{
            NSLog(@"writeData error %u",stat);
            break;
        }
    }
    return ret;
}

//////////////////////////////////////////////////////////////
//  Streamのイベント
//////////////////////////////////////////////////////////////
-(void)stream:(NSStream*)stream handleEvent:(NSStreamEvent)eventCode
{
    switch(eventCode){
        case NSStreamEventHasBytesAvailable:    //なにか受信していた時
            if(stream == self.readStream){
                [self readBuffer];
            }
            break;
        case NSStreamEventHasSpaceAvailable:
            //
            break;
        case NSStreamEventNone:
            //
            break;
        case NSStreamEventErrorOccurred:
            //
            break;
        case NSStreamEventEndEncountered:
            //
            break;
        case NSStreamEventOpenCompleted:
            //
            break;
    }
}

//////////////////////////////////////////////////////////////
//  受信データ読み込み
//////////////////////////////////////////////////////////////
-(void)readBuffer
{
    uint8_t buf[1024];
    unsigned int length = [self.readStream read:buf maxLength:1024];
    NSMutableData *data = [[NSMutableData alloc] initWithLength:length];
    [data appendBytes:buf length:length];    //ここまででdataの中に受信済みデータが入る
    //ViewControllerにイベントを渡す
    if( [self.delegate respondsToSelector:@selector(WiFiDataReceived:)] ){ //ViewControllerでメソッドが存在するかチェック
        [self.delegate WiFiDataReceived:data];  //デリゲートで受信データを渡す
    }
    
}

@end
